@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 #
 # A ridiculously simple formatter for SimpleCov results.
 #
@@ -6,7 +8,7 @@ module SimpleCov
     class SimpleFormatter
       # Takes a SimpleCov::Result and generates a string out of it
       def format(result)
-        output = ""
+        output = "".dup
         result.groups.each do |name, files|
           output << "Group: #{name}\n"
           output << "=" * 40
