@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 if defined?(JRUBY_VERSION) && JRUBY_VERSION.to_f < 1.7
   require "jruby"
   java_import "org.jruby.ast.NodeType"
@@ -11,7 +13,7 @@ if defined?(JRUBY_VERSION) && JRUBY_VERSION.to_f < 1.7
   # This monkey patches Coverage to address those issues
   module Coverage
     class << self
-      alias_method :__broken_result__, :result
+      alias __broken_result__ result
 
       def result # rubocop:disable Metrics/MethodLength
         fixed = {}
